package main

import (
	"fmt"
	"io"
	"strings"
)

func main() {
	data := []byte{1, 2, 3}
	// Badwriter(-1) 返回的是一个接口类型, 它的动态类型是(*strings.Builder, nil)
	// write 方法的参数是一个 io.Writer 类型的值, 所以实际上是是一个接口类型
	// w接受的到的动态类型是(*strings.Builder, nil)
	// 因为 (*strings.Builder, nil) 和 (nil, nil) 是不同的类型, 所以这里w != nil 为 true
	// 又因为空指针不可以调用方法, 所以这里会报错
	w := BadWriter(-1)
	fmt.Printf("w:(%T, %[1]v)\n", w) // w:(io.Writer, <nil>)
	if w != nil {
		w.Write(data)
	} else {
		println("w is nil")
	}

}

// Badwriter 接受一个 int 类型的参数，返回一个 io.Writer 类型的值
// 如果 n > 0，返回一个 *strings.Builder 类型的值
// 如果 n <= 0，返回 nil
func BadWriter(n int) io.Writer {
	// strings.Builder 是一个结构体, 它实现了 io.Writer 接口
	var b *strings.Builder = nil
	// 看看 b 的类型
	fmt.Printf("b:(%T, %[1]v)\n", b) // b:(*strings.Builder, <nil>)
	if n > 0 {
		b = &strings.Builder{}
	}
	// 当n不合法是, 返回的实际上是一个接口类型, 它的动态类型是(*strings.Builder, nil)
	return b
}

func goodWriter(n int) io.Writer {
	if n > 0 {
		return &strings.Builder{}
	}
	return nil
}
